/*
  New.cpp

  Implementation of the New function.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>

#include "TGAFile.h"

/*
int New(const int nWidth, const int nHeight)

Open TGA file

Parameters:
szFilename           const char*     Name of input file

Return value:
0 if no error occurred
a non-zero value if an error occurred
*/

int TGAFile::New(const int nWidth, const int nHeight)
{
  long lWidthInBytes;
  int x, y;
  RGBQUAD Color;

  m_pHeader = new TGAFILEHEADER;
  if(m_pHeader == 0)
  {
    fprintf(stderr, "TGAFile::Open(): Cannot allocate memory for TGAFILEHEADER.\n");
    return 3;
  }
  m_pHeader->ident_len = 0;
  m_pHeader->cmap_type = nopalette;
  m_pHeader->image_type = uncompr_rgb;
  m_pHeader->cmap_origin = 0;
  m_pHeader->cmap_len = 0;
  m_pHeader->centry_size = 24;
  m_pHeader->image_xorg = 0;
  m_pHeader->image_yorg = 0;
  m_pHeader->image_width = nWidth;
  m_pHeader->image_height = nHeight;
  m_pHeader->pixel_size = 24;
  m_pHeader->image_discr.AttBits = 0;
  m_pHeader->image_discr.IntrLve = 0;
  m_pHeader->image_discr.OrgBit = 0;
  m_pHeader->image_discr.Rsrvd = 0;

  /*
    Allocate memory for DIB
    DIB lines are DWORD aligned in memory
  */

  m_nBytesPerPixel = m_pHeader->pixel_size / 8;
  lWidthInBytes = m_nBytesPerPixel * m_pHeader->image_width;
  m_dwWidthDWORDAligned = ((lWidthInBytes - 1) / 4 + 1) * 4;

  m_pData = new unsigned char[m_dwWidthDWORDAligned * m_pHeader->image_height];

  if(m_pData == 0)
  {
    fprintf(stderr, "TGAFile::Open(): Cannot allocate memory for TGA data.\n");
    return 6;
  }

  Color.rgbBlue     = 255;
  Color.rgbGreen    = 255;
  Color.rgbRed      = 255;
  Color.rgbReserved =   0;

  for(y = 0; y < m_pHeader->image_height; y++)
  {
    for(x = 0; x < m_pHeader->image_width; x++)
    {
      SetColor(x, y, &Color);
    }
  }

  /* Go read the bits. */
  return 0;
}
