// geoDataClientDialog.h: header file
//

#pragma once
#include "afxwin.h"
#include "afxcmn.h"

#include "..\..\TGALib\inc\TGAFile.h"

#define MAXCOLORMAPSIZE        256
#define LOCALCOLORMAP          0x80
#define BitSet(byte, bit)      (((byte) & (bit)) == (bit))
#define INTERLACE              0x40
#define CM_RED                 0
#define CM_GREEN               1
#define CM_BLUE                2
#define MAX_LWZ_BITS           12

static struct
{
  int     transparent;
  int     delayTime;
  int     inputFlag;
  int     disposal;
} Gif89 = { -1, -1, -1, 0 };

typedef enum
{
  FileFormatMonochrome,
  FileFormatGrayscale,
  FileFormatColor
} tFileFormat;

typedef struct
{
  unsigned int    Width;
  unsigned int    Height;
  unsigned char   ColorMap[3][MAXCOLORMAPSIZE];
  unsigned int    BitPixel;
  unsigned int    ColorResolution;
  unsigned int    Background;
  unsigned int    AspectRatio;
  tFileFormat     FileFormat;
} GIFSCREEN;

// CgeoDataClientDialog dialog
class CgeoDataClientDialog : public CDialog
{
// Construction
public:
	CgeoDataClientDialog(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_GEODATACLIENT_DIALOG };

  bool bCancel;

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

// Implementation
protected:
	HICON m_hIcon;
  static UINT GeoCoderQueryThread(LPVOID pParam); // The static thread function. Calls:
  UINT GeoCoderQuery();                           // The non-static thread function.

  static UINT SeamlessQueryThread(LPVOID pParam); // The static thread function. Calls:
  UINT SeamlessQuery();                           // The non-static thread function.

  int nBILRows;
  int nBILCols;

  // Convert the BIL file to a TGA file using the HDR file.
  // Returns 0 if OK.
  int BilToTga(const CString &bilFilename,
               const CString &hdrFilename,
               const CString &stxFilename,
               const CString &tgaFilename);
  // Convert a GIF file to a TGA file, scale 1:1
  // Returns 0 if OK.
  int GifToTga(const CString &gifFilename,
               const CString &tgaFilename);
  int ReadGIFFile(const char *szFilename,
                  int nImageNumber,
                  TGAFile *pTGAFile);
  int ReadColorMap(FILE *pInputfile,
                   int nColorCount,
                   unsigned char buffer[3][MAXCOLORMAPSIZE],
                   tFileFormat *FileFormat);
  int DoExtension(FILE *pInputfile,
                  int nLabel);
  void ReadImage(FILE *pInputfile,
                 int nWidth,
                 int nHeight,
                 unsigned char cmap[3][MAXCOLORMAPSIZE],
                 tFileFormat FileFormat,
                 int interlace,
                 int ignore,
                 TGAFile *pTGAFile);
  int GetDataBlock(FILE *pInputfile,
                   unsigned char *pBuffer);
  int LWZReadByte(FILE *pInputfile,
                  int nFlag,
                  int nInputCodeSize);
  int GetCode(FILE *pInputfile,
              int nCodeSize,
              int nFlag);

  int            gl_ZeroDataBlock;
  int            gl_nShowComment;  // flag to show comments
  int            gl_nVerbose;      // flag for verbose output

  // Call tiger.census.gov to get the texture GIF file.
  // Returns 0 if OK.
  int TigerQuery(const CString &gifFilenameTexture,
                 double left,
                 double right,
                 double top,
                 double bottom,
                 int imagesize);
  // Convert elevation to color using a palette [file].
  // Returns 0 if OK.
  int ElevationToColor(const CString &tgaFilenameElevation,
                       const CString &tgaFilenameElevationColor);
  // Combine two TGA files. Use the size of the first one.
  // Returns 0 if OK.
  int CombineTga(const CString &tgaFilename1,
                 const CString &tgaFilename2,
                 const CString &tgaFilenameCombined);
  // Add waypoints to a TGA file. Use a predefined waypoint list.
  // Returns 0 if OK.
  int AddWaypoints(const CString &tgaFilenameTextureCombined,
                   const CString &tgaFilenameTextureCombinedWaypoints,
                   double left,
                   double top,
                   double right,
                   double bottom,
                   std::list<waypoint> waypointList);
  // Stretch/Shrink two TGA files.
  // Returns 0 if OK.
  int StretchTga(const CString &tgaFilenameInput,
                 const CString &tgaFilenameOutput,
                 double x1_, double y1_,
                 double x2_, double y2_,
                 double x3_, double y3_,
                 double x4_, double y4,
                 int nWidth, int nHeight);
  int GaussJordan(int nDimension,
                  double *pdA,
                  double *pdX);
  // Interpolate zero elevation values.
  // Returns 0 if OK.
  int interpolateElevation(const CString &inputFilename,
                           int repeats);
  int AddTga(const CString &tgaFilenameToBeAdded,
             const CString &tgaFilenameOriginal,
             int xOffset,
             int yOffset,
             int finalWidth,
             int finalHeight);
  bool readWaypoints(const CString &pathName);
  int WriteDemoTga(TGAFile &tgaFile);


  int maxListboxTextExtent;
  void BlinkError();
  void SetStateError();
  void SetStateOK();
  void SetStateWarn();
  std::list<waypoint> waypointList;
  bool allowedToUpdate; // If set to true, then updating the edit controls using UpdateData(FALSE) is permitted.

// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()
public:
  afx_msg void OnBnClickedGeocode();
  CListBox listBox;
  CButton closeButton;
  afx_msg void OnBnClickedClose();
  afx_msg void OnBnClickedCancel();
  CButton cancelButton;
  CButton proceedButton;
  bool allowedToEnableProceedButton;
  CString strAddress;
  double quadSize;
  bool interpolate;
  bool alignQuadToGrid;
  bool removeIntermediateFiles;
  int lineThickness;
  double zScale;
  int nHTTPTimeout;
  BOOL bUseWaypoints;
  BOOL outputFileNamesFromCoordinates;
  CString strWaypointFile;
  CString queryResult;
  void FilterGeocoderUS(CString strResultText);
  void addMessage(const CString& text);
  afx_msg void OnLbnSelchangeList();
  afx_msg void OnBnClickedProceed();
  CProgressCtrl progressControl;
  afx_msg void OnBnClickedCopy();
  afx_msg void OnBnClickedSettings();
  CStatic stateIndicator;
  double quadLeft;
  double quadRight;
  double quadTop;
  double quadBottom;
  afx_msg void OnEnChangeEditLeft();
  afx_msg void OnEnChangeEditTop();
  afx_msg void OnBnClickedButtonGeocoderUs();
  afx_msg void OnBnClickedButtonUsgsGov();
  afx_msg void OnBnClickedButtonCensusGov();
};
