// SettingsDialog.cpp : implementation file
//

#include "stdafx.h"

#include <list>

#include "geoDataClient.h"
#include "Waypoint.h"
#include "SettingsDialog.h"
#include "..\inc\settingsdialog.h"

// CSettingsDialog dialog

IMPLEMENT_DYNAMIC(CSettingsDialog, CDialog)
CSettingsDialog::CSettingsDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CSettingsDialog::IDD, pParent)
  , interpolate(FALSE)
  , alignQuadToGrid(FALSE)
  , removeIntermediateFiles(FALSE)
  , zScale(0)
  , bUseWaypoints(FALSE)
  , strWaypointFile(_T(""))
  , nHTTPTimeout(120)
  , outputFileNamesFromCoordinates(FALSE)
{
	quadSize = 1.0;
	lineThickness = 1;
}

CSettingsDialog::~CSettingsDialog()
{
}

void CSettingsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_QUADSIZE, comboQuadsize);
	DDX_Check(pDX, IDC_INTERPOLATE, interpolate);
	DDX_Check(pDX, IDC_ALIGN_QUAD_TO_GRID, alignQuadToGrid);
	DDX_Check(pDX, IDC_REMOVEFILES, removeIntermediateFiles);
	DDX_Text(pDX, IDC_ZSCALE, zScale);
	DDX_Check(pDX, IDC_USEWAYPOINTS, bUseWaypoints);
	DDX_Text(pDX, IDC_EDITWAYPOINTFILE, strWaypointFile);
	DDX_Control(pDX, IDC_BROWSEWAYPOINTFILE, buttonBrowseWaypointFile);
	DDX_Control(pDX, IDC_EDITWAYPOINTFILE, editWaypointFile);
	DDX_Control(pDX, IDC_COMBO_LINETHICKNESS, comboLineThickness);
	DDX_Text(pDX, IDC_TIMEOUT, nHTTPTimeout);
	DDV_MinMaxInt(pDX, nHTTPTimeout, 1, 300);
	DDX_Check(pDX, IDC_OUTPUTFILENAME, outputFileNamesFromCoordinates);
}

BEGIN_MESSAGE_MAP(CSettingsDialog, CDialog)
  ON_BN_CLICKED(IDOK, OnBnClickedOk)
  ON_BN_CLICKED(IDC_USEWAYPOINTS, OnBnClickedUsewaypoints)
  ON_BN_CLICKED(IDC_BROWSEWAYPOINTFILE, OnBnClickedBrowsewaypointfile)
  ON_CBN_SELCHANGE(IDC_COMBO_QUADSIZE, OnCbnSelchangeComboQuadsize)
END_MESSAGE_MAP()

BOOL CSettingsDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

  int selection;
  CString qS;

  qS.Format("%0.3f", 1.00);
  comboQuadsize.AddString(qS);
  qS.Format("%0.3f", 0.50);
  comboQuadsize.AddString(qS);
  qS.Format("%0.3f", 0.25);
  comboQuadsize.AddString(qS);
  qS.Format("%0.3f", 0.1);
  comboQuadsize.AddString(qS);
  qS.Format("%0.3f", 0.05);
  comboQuadsize.AddString(qS);
  qS.Format("%0.3f", 0.025);
  comboQuadsize.AddString(qS);
  qS.Format("%0.3f", quadSize);
  selection = comboQuadsize.FindString(-1, qS);
  comboQuadsize.SetCurSel(selection);

  qS.Format("%d",  1);
  comboLineThickness.AddString(qS);
  qS.Format("%d",  2);
  comboLineThickness.AddString(qS);
  qS.Format("%d",  4);
  comboLineThickness.AddString(qS);
  qS.Format("%d",  8);
  comboLineThickness.AddString(qS);
  qS.Format("%d", 16);
  comboLineThickness.AddString(qS);
  qS.Format("%d", lineThickness);
  selection = comboLineThickness.FindString(-1, qS);
  comboLineThickness.SetCurSel(selection);

  if(bUseWaypoints)
  {
    buttonBrowseWaypointFile.EnableWindow();
    editWaypointFile.EnableWindow();
    comboLineThickness.EnableWindow();
  }
  else
  {
    buttonBrowseWaypointFile.EnableWindow(FALSE);
    editWaypointFile.EnableWindow(FALSE);
    comboLineThickness.EnableWindow(FALSE);
  }

  UpdateData(FALSE);

  return TRUE;  // return TRUE  unless you set the focus to a control
}

// CSettingsDialog message handlers

void CSettingsDialog::OnBnClickedOk()
{
  // TODO: Add your control notification handler code here
  UpdateData();
  if(bUseWaypoints && strWaypointFile.GetLength() == 0)
  {
    CString message;
    message.Format("Error: The waypoint file name is empty.");
    AfxMessageBox(message, MB_ICONSTOP);
    return;
  }
  CString qS;
  comboQuadsize.GetWindowText(qS);
  quadSize = atof(qS);
  comboLineThickness.GetWindowText(qS);
  lineThickness = atoi(qS);
  if((nHTTPTimeout >= 1) && (nHTTPTimeout <= 300))
  {
    OnOK();
  }
}

double CSettingsDialog::getQuadsize()
{
  return quadSize;
}

void CSettingsDialog::setQuadsize(double quadSize)
{
  this->quadSize = quadSize;
}

bool CSettingsDialog::getInterpolate()
{
  if(interpolate)
  {
    return true;
  }
  else
  {
    return false;
  }
}

void CSettingsDialog::setInterpolate(bool interpolate)
{
  this->interpolate = interpolate;
}

bool CSettingsDialog::getAlign()
{
  if(alignQuadToGrid)
  {
    return true;
  }
  else
  {
    return false;
  }
}

void CSettingsDialog::setAlign(bool alignQuadToGrid)
{
  this->alignQuadToGrid = alignQuadToGrid;
}

bool CSettingsDialog::getRemoveIntermediateFiles()
{
  if(removeIntermediateFiles)
  {
    return true;
  }
  else
  {
    return false;
  }
}

void CSettingsDialog::setRemoveIntermediateFiles(bool removeIntermediateFiles)
{
  this->removeIntermediateFiles = removeIntermediateFiles;
}

int CSettingsDialog::getLineThickness()
{
  return lineThickness;
}

void CSettingsDialog::setLineThickness(int lineThickness)
{
  this->lineThickness = lineThickness;
}

double CSettingsDialog::getZScale()
{
  return zScale;
}

void CSettingsDialog::setZScale(double zScale)
{
  this->zScale = zScale;
}

int CSettingsDialog::getHTTPTimeout()
{
  return nHTTPTimeout;
}

void CSettingsDialog::setHTTPTimeout(int nHTTPTimeout)
{
  this->nHTTPTimeout = nHTTPTimeout;
}

BOOL CSettingsDialog::getUseWaypoints()
{
  return bUseWaypoints;
}

void CSettingsDialog::setUseWaypoints(BOOL u)
{
  bUseWaypoints = u;
}

CString CSettingsDialog::getWaypointFile()
{
  return strWaypointFile;
}

void CSettingsDialog::setWaypointFile(const CString &waypointFilename)
{
  strWaypointFile = waypointFilename;
}

BOOL CSettingsDialog::getOutputFileNamesFromCoordinates()
{
	return outputFileNamesFromCoordinates;
}

void CSettingsDialog::setOutputFileNamesFromCoordinates(BOOL o)
{
	outputFileNamesFromCoordinates = o;
}

void CSettingsDialog::OnBnClickedUsewaypoints()
{
  // TODO: Add your control notification handler code here
  UpdateData();
  if(bUseWaypoints)
  {
    buttonBrowseWaypointFile.EnableWindow();
    editWaypointFile.EnableWindow();
    comboLineThickness.EnableWindow();
  }
  else
  {
    buttonBrowseWaypointFile.EnableWindow(FALSE);
    editWaypointFile.EnableWindow(FALSE);
    comboLineThickness.EnableWindow(FALSE);
  }
}

void CSettingsDialog::OnBnClickedBrowsewaypointfile()
{
  // TODO: Add your control notification handler code here
    char lpszDefExt[] = "*.txt";
  char lpszFileName[] = "";
        DWORD dwFlags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST;
    char lpszFilter[] = "Waypoint Files (*.txt)|*.txt|All Files (*.*)|*.*||";
         DWORD dwSize = 0;
  CFileDialog fileDialog(TRUE, lpszDefExt, lpszFileName, dwFlags, lpszFilter, this, dwSize);
  if(fileDialog.DoModal() == IDOK)
  {
    // The user selected the name of a waypoint file. Try to open it and read waypoints.
    bool bResult = readWaypoints(fileDialog.GetPathName());
    if(!bResult)
    {
      strWaypointFile = fileDialog.GetPathName();
      UpdateData(FALSE);
    }
  }
}

/*
Open the given file and read waypoints.
This function considers itself OK if it could read at least one waypoint.
A waypoint file line contains at least two white space separated entries: latitude and longitude.

latitude longitude comment

Example:

# A hash sign in the first column indicates a comment line.
34.567   112.345   The rest of a line after column 2 is considered comment or waypoint name.
34.568   112.346
*/

bool CSettingsDialog::readWaypoints(const CString &pathName)
{
  std::list<waypoint> waypointList;
  FILE *waypointFile;
  waypointFile = fopen(pathName, "r");
  if(!waypointFile)
  {
    CString message;
    message.Format("Error: File \"%s\" could not be opened.", pathName);
    AfxMessageBox(message, MB_ICONSTOP);
    return true;
  }
  char szLine[1024];
  fgets(szLine, sizeof(szLine), waypointFile);
  while(!feof(waypointFile))
  {
    double latitude = 0.0;
    double longitude = 0.0;
    // Test if it's a comment line.
    if(szLine[0] != '#')
    {
      char *szLatitude = strtok(szLine, " \t"); // Read the first column.
      if(szLatitude)
      {
        latitude = atof(szLatitude);
        char *szLongitude = strtok(NULL, " \t"); // Read the second column.
        if(szLongitude)
        {
          longitude = atof(szLongitude);
          char *szComment = strtok(NULL, "\n"); // This time the delimiter is the newline character!
          if(szComment)
          {
            while(*szComment && isspace(*szComment))
            {
              szComment++;
            }
          }
          waypoint w(latitude, longitude, szComment);
          waypointList.push_back(w);
        }
      }
    }
    fgets(szLine, sizeof(szLine), waypointFile);
  }
  if(waypointList.size() == 0)
  {
    CString message;
    message.Format("Error: No waypoints in the file\n"
                   "\"%s\".\n\n"
                   "Expected layout of a line:\n\n"
                   "latitude longitude [comment]", pathName);
    AfxMessageBox(message, MB_ICONSTOP);
    return true;
  }
  fclose(waypointFile);
  return false;
}

void CSettingsDialog::OnCbnSelchangeComboQuadsize()
{
  // TODO: Add your control notification handler code here
}
